@@ -62,7 +62,19 @@ class PDAL_DLL PipelineManager
 {
     FRIEND_TEST(json, tags);
 public:
-    PipelineManager();
+    struct ExecResult
+    {
+        ExecResult() : m_mode(ExecMode::None), m_count(0)
+        {}
+        ExecResult(ExecMode mode, point_count_t count) :
+            m_mode(mode), m_count(count)
+        {}
+
+        ExecMode m_mode;
+        point_count_t m_count;
+    };
+
+    PipelineManager(point_count_t streamLimit = 10000);
     ~PipelineManager();
 
     void setProgressFd(int fd)
@@ -99,12 +111,8 @@ public:
         Stage& parent, Options options);
     Stage& makeWriter(StageCreationOptions& ops);
 
-    // returns true if the pipeline endpoint is a writer
-    bool isWriterPipeline() const
-        { return (bool)getStage(); }
-
-    // return the pipeline reader endpoint (or nullptr, if not a reader
-    // pipeline)
+    // Return the first leaf stage of a pipeline, or nullptr if the pipeline
+    // is empty.
     Stage* getStage() const
     {
         const auto& llist = leaves();
@@ -112,11 +120,11 @@ public:
     }
 
     // Set the log to be available to stages.
-    void setLog(LogPtr& log)
-        { m_log = log; }
+    void setLog(const LogPtr& log);
 
     QuickInfo preview() const;
     void prepare() const;
+    ExecResult execute(ExecMode mode);
     point_count_t execute();
     void executeStream(StreamPointTable& table);
     void validateStageOptions() const;
@@ -150,6 +158,8 @@ private:
     std::unique_ptr<StageFactory> m_factory;
     std::unique_ptr<PointTable> m_tablePtr;
     PointTableRef m_table;
+    std::unique_ptr<FixedPointTable> m_streamTablePtr;
+    StreamPointTable& m_streamTable;
     Options m_commonOptions;
     OptionsMap m_stageOptions;
     PointViewSet m_viewSet;
